/*
 * Autor: Jiri Prochazka
 */
package logger;

import javax.microedition.io.Connector;
import javax.microedition.io.HttpConnection;
import javax.microedition.rms.RecordEnumeration;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;
import java.io.*;
import java.util.Calendar;

public class Logger implements Runnable {

    public static final int LEVEL_OFF = 0;
    public static final int LEVEL_FINE = 1;
    public static final int LEVEL_INFO = 2;
    public static final int LEVEL_WARNING = 3;
    public static final int LEVEL_SEVERE = 4;

    private static final String[] LEVELS = {"OFF", "FINE", "INFO", "WARNING", "SEVERE"};
    private static final String RMS_NAME = "LOG";

    public static final int LOG_TO_STDOUT = 0x1;
    public static final int LOG_TO_RMS = 0x2;
    public static final int LOG_TO_HTTP = 0x4;

    private int _level = LEVEL_WARNING;
    private RecordStore _rs;
    private int _logTo = LOG_TO_STDOUT;
    private String _url;
    private String _msg;

    public Logger(int level) {
        _level = level;
    }

    public int getLogTo() {
        return _logTo;
    }

    public void setLogTo(int logTo, String url) throws IllegalArgumentException {

        if ((logTo & LOG_TO_HTTP) != 0 && (url == null || url.trim().equals(""))) throw new IllegalArgumentException("URL is null or empty string");
        if ((logTo & LOG_TO_RMS) != 0)
            openLogRMS();
        else
            closeLogRMS();

        _logTo = logTo;
        _url = url;
    }

    public int getLevel() {
        return _level;
    }

    public void setLevel(int level) {
        _level = level;
    }

    public boolean isLoggable(int level) {
        return (level >= _level);
    }

    public void log(Object caller, int level, String msg) {

        if (level != LEVEL_OFF) {
            synchronized (this) {
                if (!isLoggable(level)) return;

                String log = getTimeStamp() + " " + LEVELS[level] + " [" + caller.getClass().getName() + "] : " + msg;
                _msg = log;

                if ((_logTo & LOG_TO_STDOUT) != 0) {
                    System.out.println(log);
                }

                if ((_logTo & LOG_TO_RMS) != 0) {
                    storeLogToRMS(log);
                }

                if ((_logTo & LOG_TO_HTTP) != 0) {
                    Thread thread = new Thread(this);
                    thread.run();
                }
            }
        }
    }

    public void run() {

        HttpConnection con = null;

        try {
            con = (HttpConnection) Connector.open("http://" + _url);
            con.setRequestMethod(HttpConnection.POST);
            con.setRequestProperty("Content-Type", "plain\text");
            OutputStream os = con.openOutputStream();
            os.write(new String(_msg).getBytes());
            os.close();
        } catch (IOException e) {
        } finally {
            if (con != null) {
                try {
                    con.close();
                } catch (IOException e) {
                }
            }
        }
    }

    public void fine(Object caller, String msg) {
        log(caller, LEVEL_FINE, msg);
    }

    public void info(Object caller, String msg) {
        log(caller, LEVEL_INFO, msg);
    }

    public void warning(Object caller, String msg) {
        log(caller, LEVEL_WARNING, msg);
    }

    public void severe(Object caller, String msg) {
        log(caller, LEVEL_SEVERE, msg);
    }

    public String getLogsFromRMS() {
        String logs = "";
        DataInputStream dis = null;

        try {
            RecordEnumeration enum = _rs.enumerateRecords(null, null, false);

            while (enum.hasPreviousElement()) {
                dis = null;
                dis = new DataInputStream(new ByteArrayInputStream(enum.previousRecord()));
                logs += (dis.readUTF() + "\n") ;
            }
        } catch (RecordStoreException e) {
        } catch (IOException e) {
        } finally {
            if (dis != null) {
                try {
                    dis.close();
                } catch (IOException e) {
                }
            }

            dis = null;
        }

        clearLogRMS();
        closeLogRMS();

        return logs;
    }

    private void openLogRMS() {
        try {
            _rs = RecordStore.openRecordStore(RMS_NAME, true);
            clearLogRMS();
        } catch (RecordStoreException e) {
            System.out.println("Cannont open RMS!");
        }
    }

    private void closeLogRMS() {
        try {
            _rs.closeRecordStore();
        } catch (RecordStoreException e) {
            System.out.println("Cannot close RMS!");
        }

        if (_rs != null) {
            _rs = null;
        }
    }

    private void clearLogRMS() {

        try {
            RecordEnumeration enum = _rs.enumerateRecords(null, null, false);
            DataInputStream dis = null;

            while (enum.hasNextElement()) {
                _rs.deleteRecord(enum.nextRecordId());
            }
        } catch (RecordStoreException e) {
            System.out.println("Cannot clear RMS!");
        }
    }

    private void storeLogToRMS(String msg) {

        ByteArrayOutputStream buffer = new ByteArrayOutputStream();
        DataOutputStream dos = new DataOutputStream(buffer);
        byte[] data = null;

        try {
            dos.writeUTF(msg);
            data = buffer.toByteArray();

            _rs.addRecord(data, 0, data.length);

        } catch (RecordStoreException e) {
        } catch (IOException e) {
        } finally {
            if (dos != null) {
                try {
                    dos.close();
                } catch (IOException e) {
                }
            }

            if (buffer != null) {
                try {
                    buffer.close();
                } catch (IOException e) {
                }
            }

            dos = null;
            buffer = null;
            data = null;
        }
    }

    private String getTimeStamp() {

        String timeStamp = "";
        Calendar instance = Calendar.getInstance();

        timeStamp += instance.get(Calendar.DAY_OF_MONTH) + "/";
        timeStamp += (instance.get(Calendar.MONTH) + 1) + "/";
        timeStamp += instance.get(Calendar.YEAR) + "/";

        timeStamp += instance.get(Calendar.HOUR_OF_DAY) + ":";
        timeStamp += instance.get(Calendar.MINUTE) + ":";
        timeStamp += instance.get(Calendar.SECOND);

        return timeStamp;
     }
}
